## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## -----------------------------------------------------------------------------

# # Check if BioManager is installed, install if not
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# # Check if 'batchelor' is installed, install if not
# if (!requireNamespace("batchelor", quietly = TRUE)) 
#     BiocManager::install("batchelor")
# 
# # Check if 'MatrixGenerics' is installed, install if not
# if (!requireNamespace("MatrixGenerics", quietly = TRUE)) 
#     BiocManager::install("MatrixGenerics")


## -----------------------------------------------------------------------------
# devtools::install_github("yuelyu21/SCIntRuler") 

## ----setup, message=FALSE, warning=FALSE--------------------------------------
library(SCIntRuler)
library(Seurat)
library(dplyr)
library(ggplot2)

## ----load the data, message=FALSE, warning=FALSE------------------------------
# SCIntRuler::sim_data
SCIntRuler::sim_data_sce

data("sim_data_sce")
sim_data <- as.Seurat(sim_data_sce)

# labels <- c(rep("Bcell", 2000), rep("CD4T", 400),
#             rep("CD14Mono", 700), rep("CD4T", 400),
#             rep("CD56NK", 2000), rep("CD4T", 400))
# study <- c(rep("Study1", 2400),
#           rep("Study2", 1100),
#           rep("Study3", 2400))
# 
# tmpMeta <- sim_data[[]]
# tmpMeta["CellType"] <- labels
# tmpMeta["Study"] <- study
# # Add the metadata
# sim_data <- AddMetaData(sim_data, tmpMeta)
# sim_s <- subset(x = sim_data, downsample = 800)
# sim_cs <- sim_s@assays$RNA@counts[sample(32738,3000),]
# sim_data <- CreateSeuratObject(counts = sim_cs)
# sim_data <- AddMetaData(sim_data, sim_s[[]])
# sim_data <- sim_data_se

## ----show some data, message=FALSE, warning=FALSE-----------------------------
head(sim_data[[]])

## ----pre-process , message=FALSE, warning=FALSE-------------------------------
# Normalize the data
sim_data <- NormalizeData(sim_data)
# Identify highly variable features
sim_data <- FindVariableFeatures(sim_data, selection.method = "vst", nfeatures = 2000)
# Scale the data
all.genes <- rownames(sim_data)
sim_data <- ScaleData(sim_data, features = all.genes)
# Perfom linear dimensional reduction
sim_data <- RunPCA(sim_data, features = VariableFeatures(object = sim_data))
# Cluster the cells
sim_data <- FindNeighbors(sim_data, dims = 1:20)
sim_data <- FindClusters(sim_data, resolution = 0.5)
sim_data <- RunUMAP(sim_data, dims = 1:20)

## ----UMAP , message=FALSE, warning=FALSE--------------------------------------
p1 <- DimPlot(sim_data, reduction = "umap", label = FALSE, pt.size = .5, group.by = "Study", repel = TRUE)
p1

## ----UMAP2 , message=FALSE, warning=FALSE-------------------------------------

p2 <- DimPlot(sim_data, reduction = "umap", label = TRUE, pt.size = .8, group.by = "CellType", repel = TRUE)
p2


## ----CCA, message=FALSE, warning=FALSE----------------------------------------
### CCA
sim.list <- SplitObject(sim_data, split.by = "Study")
sim.anchors <- FindIntegrationAnchors(object.list = sim.list, dims = 1:30, reduction = "cca")

sim.int <- IntegrateData(anchorset = sim.anchors, dims = 1:30, new.assay.name = "CCA")
# run standard anlaysis workflow
sim.int <- ScaleData(sim.int, verbose = FALSE)
sim.int <- RunPCA(sim.int, npcs = 30, verbose = FALSE)
sim.int <- RunUMAP(sim.int, dims = 1:30, reduction.name = "umap_cca")


## ----harmony, message=FALSE, warning=FALSE------------------------------------
### Harmony
# install.packages("harmony")

sim.harmony <- harmony::RunHarmony(sim_data, group.by.vars = "Study", reduction.use = "pca",
                                   #dims.use = 1:20, assay.use = "RNA"
    )

sim.int[["harmony"]] <- sim.harmony[["harmony"]]
sim.int <- RunUMAP(sim.int, dims = 1:20, reduction = "harmony", reduction.name = "umap_harmony")


## ----reticulate, message=FALSE, warning=FALSE---------------------------------
# # create a new environment 
# reticulate::virtualenv_create("r-reticulate")
# 
# # install SciPy
# reticulate::virtualenv_install("r-reticulate", "scanorama")
# 
# reticulate::use_virtualenv("r-reticulate")
# 
# # library(reticulate)
# scanorama <- reticulate::import('scanorama')

## ----scanorama, message=FALSE, warning=FALSE----------------------------------
### Scanorama
# counts <- sim_data@assays$RNA@counts
# dim(counts)
# sim_Dat1 <- counts[,1:2400]
# sim_Dat2 <- counts[,2400+1:1100]
# sim_Dat3 <- counts[,3500+1:2400]
# 
# datasets <- list( t(as.matrix(sim_Dat1)), t(as.matrix(sim_Dat2)) ,t(as.matrix(sim_Dat3)))
# # List of gene lists:
# genes_list <- list()
# for(i in 1:3) {
#     genes_list[[i]] <- colnames(datasets[[i]])
# }
# 
# integrated.data <- scanorama$integrate(datasets, genes_list)
# 
# intdimred <- do.call(rbind, integrated.data[[1]])
# colnames(intdimred) <- paste0("PC_", 1:100)
# rownames(intdimred) <- colnames(sim.int)
# 
# # Add standard deviations in order to draw Elbow Plots in Seurat
# stdevs <- apply(intdimred, MARGIN = 2, FUN = sd)
# 
# sim.int[["scanorama"]] <- CreateDimReducObject(embeddings = intdimred, stdev = stdevs,
#     key = "scanorama_", assay = "RNA")
# 
# sim.int <- RunUMAP(sim.int, dims = 1:100, reduction = "scanorama", reduction.name = "umap_scanorama")


## ----integration, fig.height=8, fig.width=14----------------------------------
p5 <- DimPlot(sim_data, reduction = "umap", group.by = "Study") +
  theme(legend.position = "none",
        # axis.line.y = element_line( size = 2, linetype = "solid"),
        # axis.line.x = element_line( size = 2, linetype = "solid"),
        axis.text.y = element_text( color="black", size=20),
        axis.title.y = element_text(color="black", size=20),
        axis.text.x = element_text( color="black", size=20),
        axis.title.x = element_text(color="black", size=20))
p6 <- DimPlot(sim.int, reduction = "umap_cca", group.by = "Study") +
  theme(legend.position = "none",
        # axis.line.y = element_line( size = 2, linetype = "solid"),
        # axis.line.x = element_line( size = 2, linetype = "solid"),
        axis.text.y = element_text( color="black", size=20),
        axis.title.y = element_text(color="black", size=20),
        axis.text.x = element_text( color="black", size=20),
        axis.title.x = element_text(color="black", size=20))
p7 <- DimPlot(sim.int, reduction = "umap_harmony", group.by = "Study") +
  theme(legend.position = "none",
        # axis.line.y = element_line( size = 2, linetype = "solid"),
        # axis.line.x = element_line( size = 2, linetype = "solid"),
        axis.text.y = element_text( color="black", size=20),
        axis.title.y = element_text(color="black", size=20),
        axis.text.x = element_text( color="black", size=20),
        axis.title.x = element_text(color="black", size=20))
# p8 <- DimPlot(sim.int, reduction = "umap_scanorama", group.by = "Study") +
#   theme(legend.position = "none",
#         # axis.line.y = element_line( size = 2, linetype = "solid"),
#         # axis.line.x = element_line( size = 2, linetype = "solid"),
#         axis.text.y = element_text( color="black", size=20),
#         axis.title.y = element_text(color="black", size=20),
#         axis.text.x = element_text( color="black", size=20),
#         axis.title.x = element_text(color="black", size=20))

leg <- cowplot::get_legend(p5)
gridExtra::grid.arrange(gridExtra::arrangeGrob(p5 + NoLegend() + ggtitle("Unintegrated"), 
                                               p6 + NoLegend() + ggtitle("Seurat CCA") , 
                                               p7 + NoLegend() + ggtitle("Harmony"),
                                               #p8 + NoLegend() + ggtitle("Scanorama"), 
                                               nrow = 2),
    leg, ncol = 2, widths = c(20, 5))


## ----step 1, warning=FALSE,message=FALSE--------------------------------------

sim.list <- SplitObject(sim_data, split.by = "Study")
fullcluster <- GetCluster(sim.list,50,200)

normCount <- NormData(sim.list)

#distmat <- FindNNDist(fullcluster, normCount, 20)
distmat <- FindNNDistC(fullcluster, normCount, 20)


## ----step 2, warning=FALSE,message=FALSE--------------------------------------

testres <- PermTest(fullcluster,distmat,15)
PlotSCIR(fullcluster,sim.list,testres,"")

sim_result <- list(fullcluster,normCount,distmat,testres)


## ----echo=FALSE---------------------------------------------------------------
devtools::session_info()

## ----echo=FALSE---------------------------------------------------------------

# # library(showtext)
# library(hexSticker)
# ## Loading Google fonts (http://www.google.com/fonts)
# # font_add_google("Gochi Hand", "gochi")
# # font_add_google("Bebas Neue", "bebas")
# 
# ## Automatically use showtext to render text for future devices
# # showtext_auto()
# 
# 
#   SCout <- SummCluster(fullcluster)
#   refindClust <- SCout$refindClust
#   nbroad <- SCout$nbroad
#   allrevDiff <- outres$allrevDiff
#   allP <- outres$allP
# 
# 
#   Sample <- c()
#   for (i in 1:length(allP)){
#     l <- length(allP[[i]])
#     ll <- rep(names(sim.list)[i],l)
#     Sample <- c(Sample,ll)
#   }
# 
#   res <- data.frame("P value"=unlist(allP), revDiff = na.omit(c(allrevDiff)), Sample)
#   ratio <- dim(subset(res, P.value <0.1 & revDiff >0))[1]/length(Sample)
# 
#   plot <- res %>%
#     ggplot2::ggplot() +
#     ggplot2::geom_point(ggplot2::aes(revDiff,`P.value`,color = factor(Sample)), size = 1, alpha = 0.9) +
#     ggplot2::theme_test()+
#     ggplot2::scale_color_brewer(palette= "Dark2") +
#     ggplot2::geom_hline(yintercept=0.1, linetype="dashed", color = "blue") +
#     ggplot2::geom_vline(xintercept=0, linetype="dashed", color = "red")+
#     ggplot2::ylab("P-value") + ggplot2::xlab("Within-between Cluster Relative Distance")+
#     # ggplot2::annotate("text", x = max(allrevDiff, na.rm = T), y = max(unlist(allP)/2, na.rm = T),
#     #          label= paste("=",round(ratio,2)),hjust=1, vjust= 1.5, fontface="bold", size = 8)+
#     ggplot2::guides(colour = ggplot2::guide_legend(override.aes = list(size=2)))+
#     ggplot2::theme(
#      #axis.line.y = element_line(size = 2, linetype = "solid"),
#      #axis.line.x = element_line(size = 2, linetype = "solid"),
#     axis.text.y = ggplot2::element_text( color="black", size=15, face="bold"),
#     axis.title.y = ggplot2::element_text(color="black", size=15, face="bold"),
#     axis.text.x = ggplot2::element_text( color="black", size=15, face="bold"),
#     axis.title.x = ggplot2::element_text(color="black", size=15, face="bold") )+
#     ggplot2::theme(legend.position = "none")+ theme_transparent()
# 
# sticker(plot, package="SCIntRuler", p_size=20, p_x = 1.05,p_y = 1,p_color = "#5F9EA0",
#         s_x=0.88, s_y=0.9, s_width=1.5, s_height=1.5,
#         p_fontface = "plain",
#         h_fill="#C1E1C1", h_color="#5F9EA0",dpi = 600,
#         # url = "https://github.com/yuelyu21/SCIntRuler",u_color = "grey",u_size = 4,
#         # p_family = "gochi",  spotlight = T
#         filename="inst/figures/SCIntRuler.png")


## ----echo=FALSE---------------------------------------------------------------
mat1 <- matrix(1:4, ncol = 2)
mat2 <- matrix(5:8, ncol = 2)
dist_matrix <- crossdist(mat1, mat2)
#dist_matrix <- crossdist2(mat1, mat2)

## ----echo=FALSE---------------------------------------------------------------
# cell_names <- colnames(sim_data)
# set.seed(123)  # Setting a seed for reproducibility
# selected_cells <- sample(cell_names, size = 2000)
# sim_data <- subset(sim_data, cells = selected_cells)

## ----echo=FALSE---------------------------------------------------------------


# library(hexSticker)
# i <- magick::image_read("~/Desktop/SCIntegration/SCIntRuler/inst/figures/ruler2.png")
# 
# 
# sticker(i, package="SCIntRuler", p_size=20, p_x = 1.05,p_y = 1.5,p_color = "#5F9EA0",
#         s_x=0.93, s_y=0.8, s_width=1.5, s_height=1.5,
#         p_fontface = "plain",
#         h_fill="#C1E1C1", h_color="#5F9EA0",
#         # url = "https://github.com/yuelyu21/SCIntRuler",u_color = "grey",u_size = 4,
#         # p_family = "gochi",  spotlight = T
#         filename="inst/figures/SCIntRuler2.png")

